@@ -7,7 +7,7 @@ import tokenize
 from typing import Callable, Dict, List, Optional, Type
 
 import docstring_parser
-from typing_extensions import get_origin
+from typing_extensions import get_origin, is_typeddict
 
 from . import _resolver, _strings
 
@@ -75,9 +75,19 @@ class _ClassTokenization:
         for i, token in enumerate(tokens[:-1]):
             if token.token_type == tokenize.NAME:
                 # Naive heuristic for field names.
+                is_first_token = True
+                for t in tokens_from_logical_line[token.logical_line]:
+                    if t == token:
+                        break
+                    if t.token_type is not tokenize.COMMENT:
+                        is_first_token = False
+                        break
+
+                if not is_first_token:
+                    continue
+
                 if (
                     tokens[i + 1].content == ":"
-                    and tokens[i] == tokens_from_actual_line[tokens[i].actual_line][0]
                     and token.content not in field_data_from_name
                 ):
                     field_data_from_name[token.content] = _FieldData(
@@ -215,11 +225,21 @@ def get_callable_description(f: Callable) -> str:
     these docstrings."""
     f, _unused = _resolver.resolve_generic_types(f)
 
-    # Ignore any default docstrings, as generated by `dataclasses.dataclass`.
-    docstring = inspect.getdoc(f)
+    # Note inspect.getdoc() causes some corner cases with TypedDicts.
+    docstring = f.__doc__
+    if (
+        docstring is None
+        and isinstance(f, type)
+        # Ignore TypedDict's __init__ docstring, because it will just be `dict`
+        and not is_typeddict(f)
+        and not _resolver.is_namedtuple(f)
+    ):
+        docstring = f.__init__.__doc__  # type: ignore
     if docstring is None:
         return ""
 
+    docstring = _strings.dedent(docstring)
+
     if dataclasses.is_dataclass(f):
         default_doc = f.__name__ + str(inspect.signature(f)).replace(" -> None", "")
         if docstring == default_doc:
