@@ -36,7 +36,21 @@ Some examples of type annotations and the desired instantiators:
 import collections
 import dataclasses
 import enum
-from typing import Any, Callable, Dict, List, Optional, Tuple, Type, TypeVar, Union
+import inspect
+import warnings
+from collections.abc import Mapping
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Hashable,
+    List,
+    Optional,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+)
 
 from typing_extensions import Annotated, Final, Literal, get_args, get_origin
 
@@ -58,13 +72,20 @@ Instantiator = Union[
 class InstantiatorMetadata:
     nargs: Optional[Union[str, int]]
     metavar: Union[str, Tuple[str, ...]]
-    choices: Optional[Tuple[Any, ...]]
+    choices: Optional[Tuple[str, ...]]
     is_optional: bool
 
 
 class UnsupportedTypeAnnotationError(Exception):
-    """Exception raised when field actions fail; this typically means that values from
-    the CLI are invalid."""
+    """Exception raised when an unsupported type annotation is detected."""
+
+
+_builtin_set = set(
+    filter(
+        lambda x: isinstance(x, Hashable),  # type: ignore
+        __builtins__.values(),  # type: ignore
+    )
+)
 
 
 def instantiator_from_type(
@@ -85,21 +106,67 @@ def instantiator_from_type(
             type_from_typevar,
         )
 
+    if typ is Any:
+        warnings.warn("Found field with type `Any`, which will be parsed as a string.")
+        return lambda arg: arg, InstantiatorMetadata(
+            nargs=None,
+            metavar="ANY",
+            choices=None,
+            is_optional=False,
+        )
+
     # Address container types. If a matching container is found, this will recursively
     # call instantiator_from_type().
     container_out = _instantiator_from_container_type(typ, type_from_typevar)
     if container_out is not None:
         return container_out
 
-    # Construct instantiators for raw types.
+    # Validate that typ is a `(arg: str) -> T` type converter, as expected by argparse.
+    if typ in _builtin_set:
+        pass
+    elif not callable(typ):
+        raise UnsupportedTypeAnnotationError(
+            f"Expected {typ} to be a `(arg: str) -> T` type converter, but is not"
+            " callable."
+        )
+    else:
+        param_count = 0
+        has_var_positional = False
+        signature = inspect.signature(typ)
+        for i, param in enumerate(signature.parameters.values()):
+            if i == 0 and param.annotation not in (str, inspect.Parameter.empty):
+                raise UnsupportedTypeAnnotationError(
+                    f"Expected {typ} to be a `(arg: str) -> T` type converter, but got"
+                    f" {signature}. You may have a nested type in a container, which is"
+                    " unsupported."
+                )
+            if param.kind is inspect.Parameter.VAR_POSITIONAL:
+                has_var_positional = True
+            elif param.default is inspect.Parameter.empty and param.kind is not (
+                inspect.Parameter.VAR_KEYWORD
+            ):
+                param_count += 1
+
+        # Raise an error if parameters look wrong.
+        if not (param_count == 1 or (param_count == 0 and has_var_positional)):
+            raise UnsupportedTypeAnnotationError(
+                f"Expected {typ} to be a `(arg: str) -> T` type converter, but got"
+                f" {signature}. You may have a nested type in a container, which is"
+                " unsupported."
+            )
+
+    # Special case `choices` for some types, as implemented in `instance_from_string()`.
     auto_choices: Optional[Tuple[str, ...]] = None
     if typ is bool:
         auto_choices = ("True", "False")
-    elif issubclass(typ, enum.Enum):
+    elif isinstance(typ, type) and issubclass(typ, enum.Enum):
         auto_choices = tuple(x.name for x in typ)
+
     return lambda arg: _strings.instance_from_string(typ, arg), InstantiatorMetadata(
         nargs=None,
-        metavar=typ.__name__.upper(),
+        metavar=typ.__name__.upper()
+        if auto_choices is None
+        else "{" + ",".join(map(str, auto_choices)) + "}",
         choices=auto_choices,
         is_optional=False,
     )
@@ -132,7 +199,10 @@ def _instantiator_from_container_type(
             container_type = list
 
         make, inner_meta = _instantiator_from_type_inner(
-            contained_type, type_from_typevar
+            contained_type,
+            type_from_typevar,
+            allow_sequences=False,
+            allow_optional=False,
         )
         return lambda strings: container_type(
             [make(x) for x in strings]
@@ -156,7 +226,10 @@ def _instantiator_from_container_type(
             (contained_type,) = typeset_no_ellipsis
 
             make, inner_meta = _instantiator_from_type_inner(
-                contained_type, type_from_typevar
+                contained_type,
+                type_from_typevar,
+                allow_sequences=False,
+                allow_optional=False,
             )
             return lambda strings: tuple(
                 [make(x) for x in strings]
@@ -170,7 +243,12 @@ def _instantiator_from_container_type(
         else:
             instantiators, metas = zip(
                 *map(
-                    lambda t: _instantiator_from_type_inner(t, type_from_typevar),
+                    lambda t: _instantiator_from_type_inner(
+                        t,
+                        type_from_typevar,
+                        allow_sequences=False,
+                        allow_optional=False,
+                    ),
                     types,
                 )
             )
@@ -200,7 +278,10 @@ def _instantiator_from_container_type(
             )
         (typ,) = options - {type(None)}
         instantiator, metadata = _instantiator_from_type_inner(
-            typ, type_from_typevar, allow_sequences=True
+            typ,
+            type_from_typevar,
+            allow_sequences=True,
+            allow_optional=False,
         )
         return instantiator, dataclasses.replace(metadata, is_optional=True)
 
@@ -215,14 +296,62 @@ def _instantiator_from_container_type(
         if issubclass(contained_type, enum.Enum):
             choices = tuple(map(lambda x: x.name, choices))
         instantiator, metadata = _instantiator_from_type_inner(
-            contained_type, type_from_typevar
+            contained_type,
+            type_from_typevar,
+            allow_sequences=False,
+            allow_optional=False,
         )
         assert (
             # Choices provided by the contained type
             metadata.choices is None
             or len(set(choices) - set(metadata.choices)) == 0
         )
-        return instantiator, dataclasses.replace(metadata, choices=choices)
+        return instantiator, dataclasses.replace(
+            metadata,
+            choices=tuple(map(str, choices)),
+            metavar="{" + ",".join(map(str, choices)) + "}",
+        )
+
+    # Dictionaries.
+    if type_origin in (dict, Mapping):
+        key_type, val_type = get_args(typ)
+        key_instantiator, key_metadata = _instantiator_from_type_inner(
+            key_type,
+            type_from_typevar,
+            allow_sequences=False,
+            allow_optional=False,
+        )
+        val_instantiator, val_metadata = _instantiator_from_type_inner(
+            val_type,
+            type_from_typevar,
+            allow_sequences=False,
+            allow_optional=False,
+        )
+
+        def dict_instantiator(strings: List[str]) -> Any:
+            out = {}
+            if len(strings) % 2 != 0:
+                raise ValueError("incomplete set of key value pairs!")
+            for i in range(len(strings) // 2):
+                k = strings[i * 2]
+                v = strings[i * 2 + 1]
+                if key_metadata.choices is not None and k not in key_metadata.choices:
+                    raise ValueError(
+                        f"invalid choice: {k} (choose from {key_metadata.choices}))"
+                    )
+                if val_metadata.choices is not None and v not in val_metadata.choices:
+                    raise ValueError(
+                        f"invalid choice: {v} (choose from {val_metadata.choices}))"
+                    )
+                out[key_instantiator(k)] = val_instantiator(v)  # type: ignore
+            return out
+
+        return dict_instantiator, InstantiatorMetadata(
+            nargs="+",
+            metavar=f"{key_metadata.metavar} {val_metadata.metavar}",
+            choices=None,
+            is_optional=False,
+        )
 
     raise UnsupportedTypeAnnotationError(  # pragma: no cover
         f"Unsupported type {typ} with origin {type_origin}"
@@ -232,8 +361,8 @@ def _instantiator_from_container_type(
 def _instantiator_from_type_inner(
     typ: Type,
     type_from_typevar: Dict[TypeVar, Type],
-    allow_sequences: bool = False,
-    allow_optional: bool = False,
+    allow_sequences: bool,
+    allow_optional: bool,
 ) -> Tuple[Instantiator, InstantiatorMetadata]:
     """Thin wrapper over instantiator_from_type, with some extra asserts for catching
     errors."""
