@@ -23,7 +23,7 @@ from typing import (
 
 import termcolor
 import typing_extensions
-from typing_extensions import get_args, get_origin
+from typing_extensions import get_args, get_origin, is_typeddict
 
 from . import _arguments, _docstrings, _fields, _instantiators, _resolver, _strings
 
@@ -42,32 +42,43 @@ _known_parsable_types = set(
 )
 
 
-def _is_parsable_type(typ: Any) -> bool:
-    """Heuristics for determining whether a type can be treated as a single argument.
-    This is true for built-in types like ints, strings, bools, etc, as well as
-    pathlib.Path."""
+def _is_possibly_nested_type(typ: Any) -> bool:
+    """Heuristics for determining whether a type can be treated as a 'nested type',
+    where a single field has multiple corresponding argumentsi (eg for nested
+    dataclasses or classes).
+
+    Examples of when we return False: int, str, List[int], List[str], pathlib.Path, etc.
+    """
 
     origin = get_origin(typ)
     if origin is not None:
         typ = origin
 
+    # Nested types should be callable.
+    if not callable(typ):
+        return False
+
     # Known parsable types: builtins like int/str/float/bool, collections, annotations.
     if typ in _known_parsable_types:
-        return True
+        return False
 
     # Simple heuristic: dataclasses should be treated as nested objects and are not
     # parsable.
     if dataclasses.is_dataclass(typ):
-        return False
+        return True
 
-    # Non-parsable types like nested (data)classes should have fully type-annotated
-    # inputs. If any inputs are unannotated (for example, in the case of pathlib.Path),
-    # we can assume the type is parsable.
+    # TypedDict types can be unpacked.
+    if is_typeddict(typ):
+        return True
+
+    # Nested types like nested (data)classes should have fully type-annotated inputs. If
+    # any inputs are unannotated (for example, in the case of pathlib.Path), we can
+    # assume the type is not nested.
     for param in inspect.signature(typ).parameters.values():
         if param.annotation is inspect.Parameter.empty:
-            return True
+            return False
 
-    return False
+    return True
 
 
 @dataclasses.dataclass(frozen=True)
@@ -97,6 +108,9 @@ class ParserSpecification:
                     type_from_typevar[typevar] = parent_type_from_typevar[typ]  # type: ignore
 
         # Cycle detection.
+        #
+        # Note that 'parent' here refers to in the nesting hierarchy, not the
+        # superclass.
         if f in parent_classes:
             raise _instantiators.UnsupportedTypeAnnotationError(
                 f"Found a cyclic dataclass dependency with type {f}."
@@ -110,11 +124,10 @@ class ParserSpecification:
         args = []
         helptext_from_nested_class_field_name = {}
         subparsers = None
-        field_list = _fields.field_list_from_callable(
-            f=f, default_instance=default_instance
-        )
-        for field in field_list:
 
+        for field in _fields.field_list_from_callable(
+            f=f, default_instance=default_instance
+        ):
             field = dataclasses.replace(
                 field,
                 typ=type_from_typevar.get(field.typ, field.typ),  # type: ignore
@@ -131,7 +144,6 @@ class ParserSpecification:
                     " the mro: https://github.com/python/typing/issues/777"
                 )
 
-            get_origin(field.typ)
             # (1) Handle Unions over callables; these result in subparsers.
             subparsers_attempt = SubparsersSpecification.from_field(
                 field,
@@ -146,35 +158,45 @@ class ParserSpecification:
                 subparsers = subparsers_attempt
                 continue
 
-            # (2) Handle primitive types. These produce a single argument!
-            if _is_parsable_type(field.typ):
-                args.append(
-                    _arguments.ArgumentDefinition.from_field(field, type_from_typevar)
+            # (2) Handle nested callables.
+            if _is_possibly_nested_type(field.typ):
+                nested_parser = ParserSpecification.from_callable(
+                    field.typ,
+                    description=None,
+                    parent_classes=parent_classes,
+                    parent_type_from_typevar=type_from_typevar,
+                    default_instance=field.default,
                 )
+                for arg in nested_parser.args:
+                    args.append(
+                        dataclasses.replace(
+                            arg,
+                            prefix=field.name
+                            + _strings.NESTED_FIELD_DELIMETER
+                            + arg.prefix,
+                        )
+                    )
+                for k, v in nested_parser.helptext_from_nested_class_field_name.items():
+                    helptext_from_nested_class_field_name[
+                        field.name + _strings.NESTED_FIELD_DELIMETER + k
+                    ] = v
+
+                if field.helptext is not None:
+                    helptext_from_nested_class_field_name[field.name] = field.helptext
+                else:
+                    helptext_from_nested_class_field_name[
+                        field.name
+                    ] = _docstrings.get_callable_description(
+                        _resolver.resolve_generic_types(field.typ)[0]
+                    )
                 continue
 
-            # (3) Handle nested callables.
-            nested_parser = ParserSpecification.from_callable(
-                field.typ,
-                description=None,
-                parent_classes=parent_classes,
-                parent_type_from_typevar=type_from_typevar,
-                default_instance=field.default,
-            )
-            for arg in nested_parser.args:
-                args.append(
-                    dataclasses.replace(
-                        arg,
-                        prefix=field.name
-                        + _strings.NESTED_DATACLASS_DELIMETER
-                        + arg.prefix,
-                    )
+            # (3) Handle primitive types. These produce a single argument!
+            args.append(
+                _arguments.ArgumentDefinition(
+                    prefix="", field=field, type_from_typevar=type_from_typevar
                 )
-            for k, v in nested_parser.helptext_from_nested_class_field_name.items():
-                helptext_from_nested_class_field_name[
-                    field.name + _strings.NESTED_DATACLASS_DELIMETER + k
-                ] = v
-            helptext_from_nested_class_field_name[field.name] = field.helptext
+            )
 
         return ParserSpecification(
             description=description
@@ -228,7 +250,7 @@ class ParserSpecification:
                 arg.add_argument(positional_group)
                 continue
 
-            if arg.required:
+            if arg.lowered.required:
                 target_groups, other_groups = (
                     required_group_from_prefix,
                     optional_group_from_prefix,
@@ -242,7 +264,7 @@ class ParserSpecification:
             if arg.prefix not in target_groups:
                 nested_field_name = arg.prefix[:-1]
                 target_groups[arg.prefix] = parser.add_argument_group(
-                    format_group_name(nested_field_name, required=arg.required),
+                    format_group_name(nested_field_name, required=arg.lowered.required),
                     # Add a description, but only to the first group for a field.
                     description=self.helptext_from_nested_class_field_name[
                         nested_field_name
@@ -295,7 +317,9 @@ class SubparsersSpecification:
         # We don't use sets here to retain order of subcommands.
         options = [type_from_typevar.get(typ, typ) for typ in get_args(field.typ)]
         options_no_none = [o for o in options if o != type(None)]  # noqa
-        if len(options_no_none) < 2 or any(map(_is_parsable_type, options_no_none)):
+        if len(options_no_none) < 2 or not all(
+            map(_is_possibly_nested_type, options_no_none)
+        ):
             return None
 
         parser_from_name: Dict[str, ParserSpecification] = {}
